/**                                                                     // -o-
 * debug.pde
 *
 *   Debug and/or data dump functions.
 *
 *
 *   void logln(String loglevel, String s)
 *     ERRORLN(String s), WARNLN(String s), DEBUGLN(String s), NOTELN(String s)
 *     ASSERTLN(String s), ASSERT(boolean vaiid, String s)
 *
 *   String  dumpOsc( OscMessage msg, String oscpath, 
 *                    OscBundle bundle, int bundleSize ) 
 *   String  dumpOscBundle(OscBundle bundle, int bsize)
 *   String  dumpOscMessage(OscMessage msg, boolean prepend_path)
 *
 *   void  dumpSystemValues()
 *
 *   void  dumpRGB(int x, int y, int pix, int pix_prev) 
 *
 * ----------------------------------------------------------------------
 * Copyright © 2011, David Reeder.  All rights reserved.
 * Please see the LICENSE file in this distribution for license details.
 * ----------------------------------------------------------------------
 */


// ------------------------------------------------ -o--
//
void logln(String loglevel, String s) {
  println( loglevel +"  "+ s);
}

void ERRORLN(String s)  { logln("ERROR", s); exit(); }
void WARNLN(String s)  	{ logln("WARNING", s); }
void DEBUGLN(String s)  { if (1 == debug) { logln("DEBUG", s); } }
void NOTELN(String s)  	{ logln("NOTE", s); }

void ASSERTLN(String s)  { logln("ASSERT", s); exit(); }
void ASSERT(boolean valid, String s)  { if (!valid) { ASSERTLN(s); } }
 


// ------------------------------------------------ -o-
// dump osc bundles n messages
//
String dumpOsc( OscMessage      msg, 
                String          oscpath, 
                OscBundle       bundle, 
                int             bundleSize ) 
{ 
  if (null != bundle) { 
    return dumpOscBundle(bundle, bundleSize); 

  } if (null != msg) {
    return dumpOscMessage(msg, (! oscpath.equals(msg.addrPattern())));

  } else {
    WARNLN("dumpOsc(): both msg and bundle are null.");
    return "";
  }
}


String dumpOscBundle(OscBundle bundle, int bsize)
{
  String result = "";           // XXX  report time tag

  for (int i=0; i < bsize; i++) 
  {
    if ("" != result)  { result += ";  "; }
    result += dumpOscMessage(bundle.getMessage(i), true);
  }

  return result;
}


String dumpOscMessage(OscMessage msg, boolean prepend_path)
{
  String args = "";
  byte[] typetag = msg.getTypetagAsBytes();

  for (int i = 0; i < typetag.length; i++ )
  {
    if ("" != args)  { args += ", "; }

    switch(typetag[i])
    {
      case 'i':  args += msg.get(i).intValue();         break;
      case 'f':  args += msg.get(i).floatValue();       break;
      case 's':  args += msg.get(i).stringValue();      break;

      default:  this.ERRORLN("dumpOscMessage(): unknown type tag.");
    }
  }

  if (prepend_path) { args = msg.addrPattern() +" "+ args; }
  return args;
} 




// ------------------------------------------------ -o--
void dumpSystemValues()
{
  OscProperties oscprop = osc.properties();

  println("\nCAMERA DEVICES--");

  for (int i = 0; i < camDevices.length; i++) {
    println("  [" + i +"]  "+ camDevices[i]);
  }
  println("\n  camDevice_index = " + camDevice_index 
  	    + "   (" + camDevices[camDevice_index] + ")" );


  println("\nOSC--");
  println("  target = " + osc_target.address() +" : "+ osc_target.port());
  println("  localhost = " + osc.ip());
  println("  listening port = " + oscprop.listeningPort());


  println("\nSYSTEM--");

  println("  fps = " + _fps);
  println("  width/height = " + _width +" / "+ _height);
  println();
  println("  colorDiffMinimum = " + _colorDiffMinimum);
  println("  pixelDiffCountRatio = " + _pixelDiffCountRatio);
  println("  initialFrameTossCount = " + _initialFrameTossCount);
  println();
  println("  displayCaptureFilename = " + _displayCaptureFilename);
  println("  movieFilename = " + _movieFilename);
  println();

  println( printToggle("  captureVideo", captureVideo) );
  println( printToggle("  debug", debug) );
  println( printToggle("  enableOsc", enableOsc) );
  println( printToggle("  movieRecord", movieRecord) );
  println( printToggle("  showDifferance", showDifferance) );
  println( printToggle("  showOverlay", showOverlay) );
  println( printToggle("  showRawInput", showRawInput) );
  println( printToggle("  visualFeedback", visualFeedback) );

  println();
}  // dumpSystemValues()




// ------------------------------------------------ -o-
// dumpRGB
//
// dump color data for pixel at mouseX/mouseY while LEFT button is pressed.
//
// INPUT: x/y coordinates to check, current and previous pixel color values
//
// OUTPUT: x/y coordinates; r/g/b difference values; r/g/b absolute
//         values (both current and previous).
//
int cr, cg, cb,         // local variables in global space for speed (?)
    pr, pg, pb;

void dumpRGB(int x, int y, int pix, int pix_prev) 
{
  if ((1 == enableDumpRGB) && mousePressed && (LEFT == mouseButton)) {
    if ((mouseX == x) && (mouseY == y)) {

      cr = (pix >> 16) & 0xff;                  // current
      cg = (pix >> 8) & 0xff;
      cb = pix & 0xff;

      pr = (pix_prev >> 16) & 0xff;             // previous
      pg = (pix_prev >> 8) & 0xff;
      pb = pix_prev & 0xff;

      println( "(" + x +","+ y + ") \t"

               +"r: " + abs(cr-pr) 
               +"\t g: "+ abs(cg-pg) 
               +"\t b: "+ abs(cb-pb) 

               + "\t" 
               + cr + "/" + pr +" -- "
               + cg + "/" + pg +" -- "
               + cb + "/" + pb 
             );
    }
  }  
}  // dumpRGB()

